c..
c..   evalcsusy.f
c..
C-{{{ header:

c.. evalcsusy.f  --  Version 2.00
c.. ------------------------------------------------------------
c.. Program: evalcsusy
c.. Authors: R. Harlander (U Karlsruhe) and M. Steinhauser (U Hamburg)
c.. Date   : August 2004
c.. v2.00  : June 2005
c.. ------------------------------------------------------------
c.. 
c.. for documentation, see
c.. 
c.. ***********
c.. "Supersymmetric Higgs production in gluon fusion at
c..  next-to-leading order"
c..  by Robert V. Harlander and Matthias Steinhauser
c..  JHEP 09 (2004) 066 [hep-ph/0409010].
c.. 
c..  and
c.. 
c.. "Pseudo-scalar Higgs production at next-to-leading order SUSY-QCD"
c..  by Robert V. Harlander and Franziska Hofmann
c..  TTP05-07, SFB/CPP-05-17, hep-ph/0507041
c.. ***********
c.. 
c.. compilation:
c.. > make
c.. 
c.. or:
c.. > f77 -c functions.f readslha.f slhablocks.f \
c..   odeint8.f rkck8.f rkqs8.f.f obj/*.f
c.. > mv gghmix*.o obj/
c.. > f77 -o evalcsusy evalcsusy.f functions.o readslha.o slhablocks.o \
c..   odeint8.o rkck8.o rkqs8.o \
c..   obj/*.o -L`cernlib -v pro kernlib,mathlib`
c.. 
c.. execution:
c.. 
c.. > ./x.evalcsusy example.in test.out
c.. 
c.. subsequent check:
c.. > diff test.out example.out
c.. 
c.. 
c.. **************************
c.. 
c.. Remarks about compilers:
c.. * The current version has been tested on several platforms
c..   using the g77 compiler (version 2.95 or higher).
c..   Problems due to long continuation lines may arise with other compilers.
c.. * Compile time ranges from a few minutes to a few hours, depending
c..   on the platform.
c.. 
c.. **************************
c.. 
c.. CHANGES:
c.. --------
c.. * v2 includes pseudo-scalar Higgs production
c.. 
c.. **************************

C-}}}
C-{{{ main 

      program evalcsusy

      implicit real*8 (a-h,k-z)
      implicit integer(i)
      integer nf
      character jfilein*60,jfnameact*60
      logical   ifilein,lpseudo
      data pi/3.141592653589793238462643d0/
      include 'common-slha.f'

      call getarg(1,jfilein)
      inquire(file=jfilein,exist=ifilein)
      if(.not.ifilein)then
        write(6,*)'ERROR: FILE ',jfilein,' DOES NOT EXIST!'
        stop
      endif

      open(unit=12,file=jfilein)
      call readpars(12)

C-{{{ Map parameters

      Mtop = mapreal8(sminputs(6),lsminputs(6),'top mass undefined. ')
      Mstop1 = mapreal8(smassfe1(6),lsmassfe1(6)
     &     ,'stop1 mass undefined. ')
      Mstop2 = mapreal8(smassfe2(6),lsmassfe2(6),
     &     'stop2 mass undefined. ')
      Mgluino = mapreal8(dabs(smassfe1(21)),lsmassfe1(21)
     &     ,'gluino mass undefined. ')
      tanb = mapreal8(minpar(3),lminpar(3),'tanb undefined. ')
      Mw = mapreal8(smassbo(24),lsmassbo(24),'Mw undefined. ')
      Mz = mapreal8(sminputs(4),lsminputs(4),'Mz undefined. ')
      alpha = mapreal8(slhaalpha,lslhaalpha,'alpha undefined. ')

c..   default is scalar Higgs:
      if (.not.lcreinint(1)) then
         creinint(1) = 0
      endif

      if (creinint(1).eq.0) then
         lpseudo = .false.
         Mh = mapreal8(smassbo(25),lsmassbo(25)
     &        ,'scalar Higgs mass undefined. ')
      elseif (creinint(1).eq.1) then
         lpseudo = .true.
         Mh = mapreal8(smassbo(36),lsmassbo(36)
     &        ,'pseudo-scalar Higgs mass undefined. ')
      else
         write(6,*) 'ERROR: scalar or pseudo-scalar?'
         stop
      endif

      if (lcreinvar(61)) then
         q01 = creinvar(61)
      else
         q01 = 0.d0
      endif
      if (lcreinvar(62)) then
         q02 = creinvar(62)
      else
         q02 = 0.d0
      endif

      muR = mapreal8(creinvar(7) * Mh,lcreinvar(7),'muR undefined. ')
      q0c = mapreal8(creinvar(6),lcreinvar(6),'q0 undefined. ')
      q0 = q0c + q01*Mstop1 + q02*Mstop2

      muSUSY = mapreal8(hmix(1),lhmix(1),'hmix undefined. ')

      beta = atan(tanb)

      salpha = dsin(alpha)
      calpha = dcos(alpha)
      sbeta = dsin(beta)
      cbeta = dcos(beta)

      calphambeta = dcos(alpha-beta)

      cthetat = mapreal8(stopmix(1,1),lstopmix(1,1)
     &     ,'stop mixing undefined. ')
      sthetat = mapreal8(stopmix(1,2),lstopmix(1,2)
     &     ,'stop mixing undefined. ')
      s2thetat = 2*sthetat*cthetat
      c2thetat = cthetat**2 - sthetat**2

      ctW = Mw/Mz
      stW = dsqrt(1.d0-ctW**2)
      c1EW = -2.d0*Mz**2/Mtop**2 * (1.d0/2.d0-2.d0/3.d0*stW**2) *
     .     dsin( alpha + beta )
      c2EW = -2.d0*Mz**2/Mtop**2 * 2.d0/3.d0*stW**2 *
     .     dsin( alpha + beta )
      c1EWmc2EW = Mtop**2*(c1EW - c2EW)
      c1EWpc2EW = Mtop**2*(c1EW + c2EW)

      mfac = (Mstop1**2 - Mstop2**2)/2.d0/Mtop**2

c..   top and stop couplings:
      if (lpseudo) then
         gth = 1.d0/tanb
         gth12m0 = mfac*s2thetat/tanb
         gth12m1 = muSUSY/Mtop*(1.d0+1.d0/tanb**2)

         gth11 = 0.d0
         gth22 = 0.d0
         gth12 = gth12m0 + gth12m1
         gth21 = - gth12

         gbh = tanb

      else
         gth = calpha/sbeta
         gbh = salpha/cbeta
         gth11ew = c1EW*cthetat**2 + c2EW*sthetat**2
         gth22ew = c1EW*sthetat**2 + c2EW*cthetat**2
         gth12ew = 1/2.d0*(c2EW - c1EW)*s2thetat
         gth21ew = gth12ew
         gth11mu = muSUSY/Mtop*calphambeta/sbeta**2*s2thetat
         gth22mu = -gth11mu
         gth12mu = muSUSY/Mtop*calphambeta/sbeta**2*c2thetat
         gth21mu = gth12mu
         gth11al = calpha/sbeta*( 2.d0 + mfac*s2thetat**2 )
         gth22al = calpha/sbeta*( 2.d0 - mfac*s2thetat**2 )
         gth12al = mfac*calpha/sbeta*s2thetat*c2thetat
         gth21al = gth12al
         
         gth11 = gth11ew + gth11mu + gth11al
         gth22 = gth22ew + gth22mu + gth22al
         gth12 = gth12ew + gth12mu + gth12al
         gth21 = gth21ew + gth21mu + gth21al
      endif

C-}}}
C-{{{ csusy:

      rnf = 5.d0

c..   pseudo-scalar coefficient functions:
      if (lpseudo) then
         c1sm1 = 1.d0
         c1susy1 = 1.d0/tanb
         c1sm2 = 0.d0
         c1sm3 = 0.d0
         c2sm1 = -1/2.d0 + 2*dlog(muR/Mtop)

         c1susy2 = ggamixnlosusy(Mtop,Mstop1,Mstop2,Mgluino,muR,
     &        q0,sthetat,alpha,tanb,
     &        muSUSY,c1EWmc2EW,c1EWpc2EW)
         
      else
c..   scalar coefficient functions:
         c1susy1 =  3.d0 * gghmixlo(Mtop,Mstop1,Mstop2,Mgluino,muR,
     &        q0,sthetat,alpha,tanb,
     &        muSUSY,c1EWmc2EW,c1EWpc2EW)
         
         c1sm1 = 1.d0
         
         c1susy2 =  3.d0 * gghmixnlosusy(Mtop,Mstop1,Mstop2,Mgluino,muR,
     &        q0,sthetat,alpha,tanb,
     &        muSUSY,c1EWmc2EW,c1EWpc2EW)
         
         c1sm2 = 11.d0/4.d0
         c1sm3 = gghqcdnnlo(Mtop,muR,rnf)

      endif
         
C-}}}
C-{{{ Write block 'HGGSUSY'

      call getarg(2,jfnameact)
      print*,'Writing ',jfnameact

      open(unit=13,file=jfnameact,status='unknown')

      write(13,103) '#--------------------------------'
      write(13,103) '# output of evalcsusy-as.f       '
      write(13,103) '#--------------------------------'
      write(13,103) 'Block HGGSUSY'
c      if (lpseudo) then
c         write(13,104) 1,1,     '[0: sc] [1: ps-sc]'
c      else
c         write(13,104) 1,0,     '[0: sc] [1: ps-sc]'
c      endif
c      write(13,102)   6,q0c,     'q0c/Mh'
c      write(13,102)  61,q01,     'q01/Mstop1'
c      write(13,102)  62,q02,     'q02/Mstop2'
c      write(13,102)   7,muR/Mh,  'muR/Mh'
      write(13,102) 101,c1sm1,  'c1SM 1-loop'
      write(13,102) 102,c1sm2,  'c1SM 2-loop'
      write(13,102) 103,c1sm3,  'c1SM 3-loop'
      if (lpseudo) then
         write(13,102) 111,c2sm1, 'c2SM 2-loop'
      endif
      write(13,102) 201,c1susy1,'c1SUSY 1-loop'
      write(13,102) 202,c1susy2,'c1SUSY 2-loop'
      write(13,102) 1001,gth,'gth'
      write(13,102) 1011,gth11,'gth11'
      write(13,102) 1022,gth22,'gth22'
      write(13,102) 1012,gth12,'gth12'
      write(13,102) 1021,gth21,'gth21'
      write(13,102) 2001,gbh,'gbh'
      call filedump(12,13)
      write(13,103) '#--------------------------------'
      write(13,103) '# END of output of evalcsusy-as.f'
      write(13,103) '#--------------------------------'

      close(12)
      close(13)

 102  format('    ',i4,'  ',e16.8,'  # ',a20)
 103  format(a)
 104  format('    ',i4,'  ',i4,'              # ',a20)

C-}}}

      end

C-}}}
C-{{{ subroutine readpars:

      subroutine readpars(iunit)
c..   
c..   Read blocks specified below from output file of 'spectrum calculator'
c..   
      implicit real*8 (a-h,k-z)
      implicit integer(i)
      character jfilein*60,jfnameact*60
      character blocks(20)*15
      include 'common-slha.f'

      do i=1,20
         blocks(i) = ' '
      enddo

      do i=1,100
         lsmassbo(i)   = .false.
         lsmassfe1(i)  = .false.
         lsmassfe2(i)  = .false.
         lsminputs(i)  = .false.
         lminpar(i)    = .false.
         lcreinvar(i)  = .false.
      enddo

      do i=1,2
         do j=1,2
            lstopmix(i,j) = .false.
         enddo
      enddo

      do i=1,10
         lcreinint(i) = .false.
      enddo
      lslhaalpha = .false.

      blocks(1) = 'CREIN'
      blocks(2) = 'MASS'
      blocks(3) = 'SMINPUTS'
      blocks(4) = 'ALPHA'
      blocks(5) = 'HMIX'
      blocks(6) = 'STOPMIX'
      blocks(7) = 'MINPAR'

      call readblocks(iunit,blocks)

      end

C-}}}
C-{{{ function b0fin:

      real*8 function b0fin(q2,m1,m2,mu)
c..
c..   This is the finite part of the B0 function.
c..   Only the real part is implemented; the imaginary part
c..   is set to zero.
c..   Note: not all cases are covered yet; corresponding 
c..   'if-endif' still missing.
c..   
c..   Is it necessary to implement also special cases (mi=0, q2=0,  ...)?
c..   
c..   q2 == q^2
c..   m1 == m_1
c..   m2 == m_2
c..

      implicit real*8 (a-z)

      mu2 = mu*mu

      if ( (q2.le.(m1-m2)**2) ) then
         b0fin = 
     .        (2.d0-dlog(m1*m2/mu2)
     .        +(m1**2-m2**2)/q2*dlog(m2/m1)
     .        + dsqrt((m1+m2)**2-q2)*dsqrt((m1-m2)**2-q2)/q2 *
     .        dlog(  
     .        (dsqrt((m1+m2)**2-q2)+dsqrt((m1-m2)**2-q2)) /
     .        (dsqrt((m1+m2)**2-q2)-dsqrt((m1-m2)**2-q2)) 
     .        ) )
      elseif ( (q2.ge.(m1+m2)**2) ) then
         b0fin = 
     .        (2.d0-dlog(m1*m2/mu2)
     .        +(m1**2-m2**2)/q2*dlog(m2/m1)
     .        - dsqrt(q2-(m1+m2)**2)*dsqrt(q2-(m1-m2)**2)/q2 *
     .        dlog(-  
     .        (dsqrt(q2-(m1+m2)**2)+dsqrt(q2-(m1-m2)**2)) /
     .        (dsqrt(q2-(m1+m2)**2)-dsqrt(q2-(m1-m2)**2)) 
     .        ) )
      elseif ( (q2.lt.(m1+m2)**2).and.(q2.gt.(m1-m2)**2) ) then
         b0fin = 
     .        (2.d0-dlog(m1*m2/mu2)
     .        +(m1**2-m2**2)/q2*dlog(m2/m1)
     .        - 2.d0/q2 *
     .        dsqrt( dabs(q2-(m1+m2)**2) )*
     .        dsqrt( dabs(q2-(m1-m2)**2) )*
     .        datan( dsqrt(dabs(q2-(m1-m2)**2)) / 
     .        dsqrt(dabs(q2-(m1+m2)**2)) )
     .        )
      else
         write(6,*) '<function b0fin>: ',
     .        'arguments (',q2,',',m1,',',m2,') not implemented.'
         stop
      endif
      end

C-}}}
C-{{{ subroutine filedump:

      subroutine filedump(iunitin,iunitout)
c..
c..   Write file unit=iunitin into file unit=iunitout.
c..   Both files must be open before calling filedump,
c..   and both files will be at EOF after calling filedump.
c..
      integer iunitin,iunitout,ipoint
      character*200 achar

      rewind(iunitin)
 201  read(iunitin,1112,END=202) achar
      length = lnblnk(achar)
      write(iunitout,1112) achar(1:length)
      goto 201
 202  continue

 1112 format(a)

      end

C-}}}
C-{{{ function gghmixlo:

      real*8 function gghmixlo(Mtop,Mstop1,Mstop2,Mgluino,muR,
     &     q0,sthetat,alpha,tanb,
     &     muSUSY,c1EWmc2EW,c1EWpc2EW)
      
      implicit real*8 (a-z)
c..   
c..   LO ggH coefficient function in the MSSM
c..   In the notation of the paper gghmixlo corresponds to
c..   1/3*( c^{(0)}_{1,t} + c^{(0)}_{1,\tilde t} )
c..   Note: C_1 = -\alpha_s/\pi * gghmixlo + h.o.
c..
      beta = datan(tanb)
      sbeta = dsin(beta)
      salpha = dsin(alpha)
      calpha = dcos(alpha)
      calphambeta = dcos(alpha-beta)
      cthetat = dsqrt(1.d0-sthetat*sthetat)

      dum1 = calpha*(1/(3.*sbeta) + Mtop**2/(12.*Mstop1**2*sbeta) + Mtop
     &     **2/(12. *Mstop2**2*sbeta) + (cthetat**2*sthetat**2)/(6.
     &     *sbeta) - (cthetat**2*Mstop1**2*sthetat**2)/(12.*Mstop2**2
     &     *sbeta) - (cthetat**2*Mstop2**2*sthetat**2)/(12.*Mstop1**2
     &     *sbeta))

      dum2 = muSUSY*( (calphambeta*cthetat*Mtop*sthetat)/(12.*Mstop1**2
     &     *sbeta**2)-(calphambeta*cthetat*Mtop*sthetat)/(12.*Mstop2**2
     &     *sbeta**2) )
      
      dum3 = c1EWpc2EW*( 1/(48.*Mstop1**2) + 1/(48.*Mstop2**2) )

      dum4 = c1EWmc2EW*(
     &     cthetat**2/(48.*Mstop1**2) - cthetat**2/(48.*Mstop2**2) - 
     &     sthetat**2/(48.*Mstop1**2) + sthetat**2/(48.*Mstop2**2) )

      gghmixlo = dum1 + dum2 + dum3 + dum4

      end

C-}}}
C-{{{ function gghmixnlosusy:

      real*8 function gghmixnlosusy(Mtop,Mstop1,Mstop2,Mgluino,muR,
     &     q0,sthetat,alpha,tanb,
     &     muSUSY,c1EWmc2EW,c1EWpc2EW)

c..
c..

      implicit real*8 (a-z)

c..   allowed difference between mass-differences:
      del = .01d0

      Mtoptmp1 = Mtop*(1.d0+15.d0*del)
      Mtoptmp2 = Mtop*(1.d0-15.d0*del)
      Mstop1tmp1 = Mstop1*(1.d0+del)
      Mstop1tmp2 = Mstop1*(1.d0-del)

      Mstop1def = Mstop1
      if ( dabs(Mstop1-Mstop2)/Mstop1.le.del ) then 
         Mstop1=Mstop2*(1.+del/10.d0)
      endif

      dumbare = 0.d0
      dumCT = 0.d0

      if ( ( (dabs(Mstop1-(Mgluino-Mtop))/Mstop1).le.del ).or.
     &     ( (dabs(Mstop2-(Mgluino-Mtop))/Mstop2).le.del ).or.
     &     ( (dabs(Mstop1-(Mgluino+Mtop))/Mstop1).le.del ).or.
     &     ( (dabs(Mstop2-(Mgluino+Mtop))/Mstop2).le.del ) ) then
         dumbare1= gghnlobare(Mtoptmp1,Mstop1,Mstop2,Mgluino,muR,
     &        q0,sthetat,alpha,tanb,
     &        muSUSY,c1EWmc2EW,c1EWpc2EW)
         dumbare2= gghnlobare(Mtoptmp2,Mstop1,Mstop2,Mgluino,muR,
     &        q0,sthetat,alpha,tanb,
     &        muSUSY,c1EWmc2EW,c1EWpc2EW)
 
         dumbare = (dumbare1+dumbare2)/2.d0
      elseif ( dabs(Mstop1-Mstop2)/Mstop1.le.del ) then 
         dumbare1= gghnlobare(Mtop,Mstop1tmp1,Mstop2,Mgluino,muR,
     &        q0,sthetat,alpha,tanb,
     &        muSUSY,c1EWmc2EW,c1EWpc2EW)
         dumbare2= gghnlobare(Mtop,Mstop1tmp2,Mstop2,Mgluino,muR,
     &        q0,sthetat,alpha,tanb,
     &        muSUSY,c1EWmc2EW,c1EWpc2EW) 

         dumbare = (dumbare1+dumbare2)/2.d0
      else
         dumbare = gghnlobare(Mtop,Mstop1,Mstop2,Mgluino,muR,
     &     q0,sthetat,alpha,tanb,
     &     muSUSY,c1EWmc2EW,c1EWpc2EW)
      endif

      dumCT =  gghmix_CT1(Mtop,Mstop1,Mstop2,Mgluino,muR,
     &     q0,sthetat,alpha,tanb,
     &     muSUSY,c1EWmc2EW,c1EWpc2EW)
     &     + gghmix_CT2(Mtop,Mstop1,Mstop2,Mgluino,muR,
     &     q0,sthetat,alpha,tanb,
     &     muSUSY,c1EWmc2EW,c1EWpc2EW)
     &     + gghmix_CT3(Mtop,Mstop1,Mstop2,Mgluino,muR,
     &     q0,sthetat,alpha,tanb,
     &     muSUSY,c1EWmc2EW,c1EWpc2EW)
     &     + gghmix_CT4(Mtop,Mstop1,Mstop2,Mgluino,muR,
     &     q0,sthetat,alpha,tanb,
     &     muSUSY,c1EWmc2EW,c1EWpc2EW)
     &     + gghmix_CT5(Mtop,Mstop1,Mstop2,Mgluino,muR,
     &     q0,sthetat,alpha,tanb,
     &     muSUSY,c1EWmc2EW,c1EWpc2EW)
     &     + gghmix_CT6(Mtop,Mstop1,Mstop2,Mgluino,muR,
     &     q0,sthetat,alpha,tanb,
     &     muSUSY,c1EWmc2EW,c1EWpc2EW)

      gghmixnlosusy = dumbare + dumCT

      Mstop1 = Mstop1def

      end

C-}}}
C-{{{ function ggamixnlosusy:

      real*8 function ggamixnlosusy(Mtop,Mstop1,Mstop2,Mgluino,muR,
     &     q0,sthetat,alpha,tanb,
     &     muSUSY,c1EWmc2EW,c1EWpc2EW)
c..
c..
      implicit real*8 (a-z)
      real*8 masses(4),mmax(4)
      integer imax(4),i,j,k,npoles

c..   allowed difference between masses:
      del = 1d-4
      delfac = 0.d0

      if ((Mgluino.lt.1.d0).or.(Mtop.lt.1.d0).or.(Mstop1.lt.1.d0).or
     &     .(Mstop2.lt.1.d0))  then
         call printdie('One of the masses is too small.')
         stop
      endif

      masses(1) = Mstop1
      masses(2) = Mstop2
      masses(3) = Mtop
      masses(4) = Mgluino

      do i=1,4
         mmax(i) = masses(i)
         imax(i) = i
      enddo

      do i=1,4
         do j=i+1,4
            if (mmax(j).gt.mmax(i)) then
               tmp = mmax(j)
               mmax(j) = mmax(i)
               mmax(i) = tmp
               tmp = imax(i)
               imax(i) = imax(j)
               imax(j) = tmp
            endif
         enddo
      enddo
      
 101  npoles = 0
      do k=1,4
         do i=k+1,4
            do j=i+1,4
C               print*,imax(k),imax(i),imax(j)
               if ( dabs((masses(imax(i))+masses(imax(j)))/masses(imax(k
     &              )) - 1.d0).lt.del )then
                  masses(imax(k)) = (masses(imax(i))+masses(imax(j)))
     &                 * (1 + delfac*del)
C                  print*,masses(imax(i)),masses(imax(j)),masses(imax(k))
C     &                 ,mmax(k)
                  npoles = npoles+1
               endif
            enddo
         enddo
      enddo
      if (npoles.gt.0) then
         delfac = delfac + 1.d0
c         print*,'delfac = ',delfac,npoles
         goto 101
      endif

      if (dabs(masses(1)/masses(2) - 1.d0).lt.del) then
         masses(1) = masses(2) * ( 1 + 2*del )
      endif


      Mstop1l = masses(1)
      Mstop2l = masses(2)
      Mtopl = masses(3)
      Mgluinol = masses(4)

      ggamixnlosusy = ggamix_bare(Mtopl,Mstop1l,Mstop2l,Mgluinol,muR,
     &     q0,sthetat,alpha,tanb,
     &     muSUSY,c1EWmc2EW,c1EWpc2EW)

      end

C-}}}
C-{{{ function gghnlobare:

      real*8 function gghnlobare(Mtop,Mstop1,Mstop2,Mgluino,muR,
     &     q0,sthetat,alpha,tanb,
     &     muSUSY,c1EWmc2EW,c1EWpc2EW)

      implicit real*8 (a-z)
c..   
c..
c..

         gghnlobare = gghmix_bare1(Mtop,Mstop1,Mstop2,Mgluino,muR,
     &     q0,sthetat,alpha,tanb,
     &     muSUSY,c1EWmc2EW,c1EWpc2EW)
     &     + gghmix_bare2(Mtop,Mstop1,Mstop2,Mgluino,muR,
     &     q0,sthetat,alpha,tanb,
     &     muSUSY,c1EWmc2EW,c1EWpc2EW)
     &     + gghmix_bare3(Mtop,Mstop1,Mstop2,Mgluino,muR,
     &     q0,sthetat,alpha,tanb,
     &     muSUSY,c1EWmc2EW,c1EWpc2EW)
     &     + gghmix_bare4(Mtop,Mstop1,Mstop2,Mgluino,muR,
     &     q0,sthetat,alpha,tanb,
     &     muSUSY,c1EWmc2EW,c1EWpc2EW)
     &     + gghmix_bare5(Mtop,Mstop1,Mstop2,Mgluino,muR,
     &     q0,sthetat,alpha,tanb,
     &     muSUSY,c1EWmc2EW,c1EWpc2EW)
     &     + gghmix_bare6(Mtop,Mstop1,Mstop2,Mgluino,muR,
     &     q0,sthetat,alpha,tanb,
     &     muSUSY,c1EWmc2EW,c1EWpc2EW)

      end

C-}}}
C-{{{ function gghqcdnnlo:

      real*8 function gghqcdnnlo(Mtop,muR,nl)

      implicit real*8 (a-z)
c..   
c..   NNLO (pure QCD) coefficient of C_1, i.e. the coefficient of
c..   api5^2 of the expression (we have nl=5):
c..
c..   1 + (11*api5)/4 + api5^2*(2777/288 + (19*lmm)/16 + (-67/96 + lmm/3)*nl)  
c..

      lmm = dlog(muR**2/Mtop**2)

      gghqcdnnlo = 2777.d0/288.d0 + (19.d0*lmm)/16.d0 
     .     + (-67.d0/96.d0 + lmm/3.d0)*nl

      end

C-}}}
C-{{{ function davytausk:

      real*8 function davytausk(xin,yin,m3in)
c..
c..   Version 2.0
c..
c..   This is the function
c..   
c..   \lambda^2(x,y) * \Phi(x,y)
c..   
c..   as defined in
c..   A.I. Davydychev and J.B. Tausk,  Nucl. Phys. B 397 (1993) 123
c..   Eq. (4.10). 
c..
c..   It uses CERNLIB for the Clausen function, and the dilogarithm from
c..   other sources.
c..   Thus, compile e.g. with
c..
c..   setenv LIBS `cernlib -v pro kernlib,mathlib`
c..   f77 -o davytausk davytausk.f ../ggah/v2.0/functions.f -L$LIBS
c..
c..   Note that the printed version of Eq.(4.10) has a typo:
c..   the arguments
c..   1/2 *  1 + x - y - \lambda
c..   and
c..   1/2 *  1 - x + y - \lambda
c..   should read
c..   1/2 * (1 + x - y - \lambda)
c..   and
c..   1/2 * (1 + x - y - \lambda)
c..
c..   
      implicit real*8 (a-z)
      real*8 m32,m3in
      data pi/3.14159265358979323846264338328d0/
      complex*16 dilog,dtphixyc

      sqx = dsqrt(xin)
      sqy = dsqrt(yin)

      dtlam2 = (1.d0 - xin - yin)**2 - 4*xin*yin

c..   First the case lambda^2 >= 0, which is given by Eq.(4.10).
c..   But before, we need to do some mapping to cover all
c..   kinematical cases (see discussion below Eq.(4.10)).
      if (dtlam2.gt.0) then
c..   m1 + m2 <= m3:
         if     ( (sqx+sqy).le.1.d0 ) then
            xx = xin
            yy = yin
            m32 = m3in**2
c..   m2 + m3 <= m1  (m3 <-> m1):
         elseif ( (sqx-sqy).gt.1.d0 ) then
            xx = 1/xin
            yy = yin/xin
            m32 = xin*m3in**2
c..   m1 + m3 <= m2  (m2 <-> m3):
         elseif ( (sqy-sqx).gt.1.d0 ) then
            xx = xin/yin
            yy = 1/yin
            m32 = yin*m3in**2
         endif

c..   the sign of lambda^2 is invariant under the mappings above, but
c..   lambda itself is not:
         dtlam2xy = (1.d0 - xx - yy)**2 - 4*xx*yy
         dtlambda = dsqrt(dtlam2xy)

c..   Eq.(4.10) times lambda^2:
         dtphixyc = 
     &        dtlambda * m32 * ( 2*dlog( dtarg(xx,yy,dtlambda) )
     &        *dlog( dtarg(yy,xx,dtlambda) )
     &        - dlog(xx)*dlog(yy)
     &        - 2*dilog( dtarg(xx,yy,dtlambda) )
     &        - 2*dilog( dtarg(yy,xx,dtlambda) )
     &        + pi*pi/3.d0 )

c..   Now the case lambda^2 < 0:
      elseif (dtlam2.lt.0) then
c..   and again the mappings, but this time   
c..   m1 + m2 >= m3  is the default:
         if     ( (sqx+sqy).ge.1.d0 ) then
            xx = xin
            yy = yin
            m32 = m3in**2
         elseif ( (sqx-sqy).lt.1.d0 ) then
            xx = 1/xin
            yy = yin/xin
            m32 = xin*m3in**2
         elseif ( (sqy-sqx).lt.1.d0 ) then
            xx = xin/yin
            yy = 1/yin
            m32 = yin*m3in**2
         endif

         dtlam2xy = (1.d0 - xx - yy)**2 - 4*xx*yy
         dtlambda = dsqrt(-dtlam2xy)

c..   Eq.(4.15) times lambda^2:
         dtphixyc = - 2.d0 * m32 * dtlambda *
     &        ( dtclaus(1.d0,xx,yy)
     &        + dtclaus(yy,xx,1.d0)
     &        + dtclaus(xx,yy,1.d0) )

      else
         dtphixyc = 0.d0
      endif
         
      if (dabs(dimag(dtphixyc)).gt.1d-10) stop 1003
      davytausk = dreal(dtphixyc)

      end

C-}}}
C-{{{ function dtclaus:

      real*8 function dtclaus(arg1,arg2,arg3)
c..
c..   This is Clausen's function with the particular argument as 
c..   defined in Eq.(4.15) of
c..   Davydychev, Tausk,  NPB 397 (1993) 123.
c..
c..   dclaus(x) is taken from CERNLIB which has to be compiled in.
c..   
      implicit real*8 (a-z)
      external dclaus

      argu = 2*acos( (- arg1 + arg2 + arg3)/2.d0/dsqrt(arg2*arg3) )
      dtclaus = dclaus(argu)

      end

C-}}}
C-{{{ function dtarg:

      real*8 function dtarg(xin,yin,xylam)

      implicit real*8 (a-z)

      dtarg = (1 + xin - yin - xylam)/2.d0

      end

C-}}}
C-{{{ function mapint:

      integer function mapint(val,lval,message)

      implicit integer(a-z)
      character(*) message
      logical lval

      if (lval) then
         mapint = val
      else
         call printdie(message)
      endif
      
      return
      end

C-}}}
C-{{{ function mapreal8:

      real*8 function mapreal8(val,lval,message)

      implicit real*8(a-z)
      character(*) message
      logical lval

      if (lval) then
         mapreal8 = val
      else
         call printdie(message)
      endif
      
      return
      end

C-}}}
C-{{{ subroutine printdie:

      subroutine printdie(strng)

      character*(*) strng

      write(6,*) 'evalcsusy.f (fatal): ',strng,' Stopped.'
      stop

      end

C-}}}





